(* Global variables that make the interpreter output easier to read. *)
Control.Print.printDepth  := 20;
Control.Print.printLength := 30;


(* Section 11.2 *)

datatype day = Mon | Tue | Wed | Thu | Fri | Sat | Sun;

fun isWeekDay x = not (x = Sat orelse x = Sun);

(* Here is a better definition for isWeekDay. *)
fun isWeekDay Sat = false
 |  isWeekDay Sun = false
 |  isWeekDay  _  = true;

 isWeekDay Mon;
 isWeekDay Sun;



(* Section 11.3 *)

(* A datatype for "extended integers". *)
datatype exint = PlusInf      (* positive infinity *)
               | MinusInf     (* negative infinity *)
               | Value of int;

PlusInf;   (* this is now an ML value of type exint *)
MinusInf;  (* this is another ML value of type exint *)
Value;     (* this is a "constructor" function that constructs values of type exint *)
Value 3;   (* this is a value of type exint *)

(* unfortunately, you cannot yet add values of type exint *)
(*
val x = Value 5;
x + x;
*)

fun explus PlusInf   PlusInf  = PlusInf
 |  explus MinusInf  MinusInf = MinusInf
 |  explus PlusInf   MinusInf = Value 0  (* not realy correct *)
 |  explus MinusInf  PlusInf  = Value 0
 |  explus (Value x) PlusInf  = PlusInf
 |  explus PlusInf  (Value x) = PlusInf
 |  explus (Value x) MinusInf = MinusInf
 |  explus MinusInf (Value x) = MinusInf
 |  explus (Value x)(Value y) = Value (x+y)
 handle Overflow => PlusInf; (* not quite correct *)

 explus (Value 4) (Value 5);
 val x = Value 5;
 val y = Value 7;
 explus x y;
 explus (Value ~1000) PlusInf;
 explus (Value 1000000000) (Value 1000000000);
(*
1000000000 + 1000000000;
*)

fun square PlusInf  = PlusInf
 |  square MinusInf = PlusInf
 |  square (Value x) = Value (x*x)
 handle Overflow => PlusInf;

square (Value 12);
square (Value 10000000);
(*
10000000 * 10000000;
*)



(* Section 11.4 *)

(* This is an example of a "parameterized type". *)
datatype 'x bunch = One of 'x
                  | Group of 'x list;

One 1.0;                     (* type "real bunch" *)
Group [true, false];         (* type "bool bunch" *)
One true;                    (* type "bool bunch" *)
Group [1, ~3, 5, 7];         (* type "int bunch" *)
Group [0.1, 0.2, 0.3, 0.4];  (* type "real bunch" *)
Group [[1,2], [3,4], [5,6]]; (* type "int list bunch" *)

(* The following function is very polymorphic.
   It works on any input of type 'a bunch. *)
fun size (One _) = 1
 |  size (Group x) = length x;

size (Group [1, ~3, 5, 7]);
size (Group [0.1, 0.2, 0.3, 0.4]);
size (One false);
size (Group [[1,2], [3,4], [5,6]]);

(* The following function is not very polymorphic.
   It only works on inputs of type int bunch.
   Why?  *)
fun sum (One x) = x
 |  sum (Group xlist) = foldr (op +) 0 xlist;

sum (One 5);
sum (Group [1,2,3,4]);
sum (One 0.2); (* will not compile *)
sum (Group [[1,2], [3,4], [5,6]]); (* will not compile *)